#ifndef EQUATION_H_
#define EQUATION_H_
#include <string>

using namespace std;


/**
 * This class is an representation of a first order equation without variables.
 * 
 * @author Patrick Bahr
 */

template<class T> class Equation
{
public:
	/**
	 * Constructs an equation.
	 */
	Equation(T* lhs, T* rhs);
	virtual ~Equation();
		
	
	/**
	 * Generates a human readable string representation of this equation.
	 */
	string toString() const;
	
	/**
	 * Provides the left hand side of this equation.
	 */
	T*const getLhs() const;
	/**
	 * Provides the right hand side of this equation.
	 */
	T*const getRhs() const;
	/**
	 * Provides the size of this equation
	 */
	
	int size() const;

private:
	/**
	 * The left hand side of this equation.
	 */
	T*const lhs;
	
	/**
	 * The right hand side of this equation.
	 */
	T*const rhs;
};

template<class T> inline int Equation<T>::size() const {
	return lhs->size() + rhs->size();
}

template<class T> inline T*const Equation<T>::getLhs() const {
	return lhs;
}

template<class T> inline T*const Equation<T>::getRhs() const {
	return rhs;
}

#endif /*EQUATION_H_*/

