#ifndef PARSEEXCEPTION_H_
#define PARSEEXCEPTION_H_
#include <string>

using namespace std;
/**
 * This exception is supposed to be thrown in case of an erroneous input to an parsing algorithm.
 * 
 * @author Patrick Bahr
 */
class ParseException
{
public:
	/**
	 * Constructs a parser related exception.
	 * @param message A message describing the exception.
	 * @param position The position in the input where the exception occurred.
	 */
	ParseException(string message ,int position);
	virtual ~ParseException();
	
	/**
	 * Provides the error message
	 */
	string getMessage() const;
	
	/**
	 * Provides the position in the input where the exception occurred.
	 */
	int getPosition() const;
	/**
	 * Generates a ready-to-use output message containig the position of the exception.
	 * 
	 * @param input the input string in which the exception occurred
	 */
	string getFullMessage(string input) const;
	/**
	 * Generates a ready-to-use output message containig the position of the exception
	 * and additionally the given information of the line in which the exception occurred.
	 * Use this method for multiline input that is parsed line by line.
	 * 
	 * @param input the input string in which the exception occurred
	 * @param line the line in which the exception occurred.
	 */
	string getFullMessage(string input,int line) const;

private:
	const string message;
	const int position;
};

inline string ParseException::getMessage() const {
	return message;
}
inline int ParseException::getPosition() const {
	return position;
}

#endif /*PARSEEXCEPTION_H_*/
