#include "SigTab.h"

SigTab::SigTab(const SignatureTable& ist)
:nextClass(0)
{
	for(SignatureTable::const_iterator it = ist.begin();it != ist.end();it++) {
		
		Signature<EqClass>* sig = it->first;
		EqClass cl = it->second;
		const Symbol* sym = sig->getSymbol();
		ArityTable::iterator ait = arityTable.find(sym->getName());
		if(ait == arityTable.end()) {
			sym = new Symbol(*sym); // copy
			arityTable.insert(make_pair(sym->getName(),sym));
		} else {
			sym = ait->second;
		}
		Signature<int>* nsig = new Signature<int>(sym);
		for(int i=0;i<sym->getArity();i++) {
			int c = (*sig)[i]->getAuxMember();
			if(c==-1) {
				c = nextClass++;
				(*sig)[i]->setAuxMember(c);
			}
			(*nsig)[i] = c;
		}
		int c = cl->getAuxMember();
		if(c==-1) {
			c = nextClass++;
			cl->setAuxMember(c);
		}
		signatureTable.insert(make_pair(nsig,c));
	}
}

SigTab::~SigTab()
{
}



Equation<Term>* SigTab::parseEquation(const string& equation) {
	int offset = 0;
	return parseEq<Term>(arityTable,equation,offset);
}
	
bool SigTab::decide(Equation<Term>* equation) {
	return findEqClass(equation->getLhs()) == findEqClass(equation->getRhs());
}

int SigTab::findEqClass(Term* t) {
	const Symbol* sym = t->getSymbol();
	Signature<int>* sig = new Signature<int>(sym);
	for(int i=0;i<sym->getArity();i++) {
		(*sig)[i] = findEqClass((*t)[i]);
	}
	IntSignatureTable::iterator it = signatureTable.find(sig);
	if(it != signatureTable.end()) {
		delete sig;
		return it->second;
	}
	signatureTable.insert(make_pair(sig,nextClass));
	return nextClass++;
}
