#ifndef SYMBOL_H_
#define SYMBOL_H_

#include <string>
#include <ext/hash_map>

using namespace std;
using namespace __gnu_cxx;

/**
 * This class represents an operation symbol in a term, consisting of a name and an arity.
 * 
 * @author Patrick Bahr
 */
class Symbol
{
public:
	/**
	 * Constructs an operation symbol representation.
	 * 
	 * @param name the name of the operation, i.e. the symbol itself
	 * @param arity of the operation, where 0 indicates a constant for example
	 */
	Symbol(const string name, const int arity);
	virtual ~Symbol();
	
	/**
	 * Provides the name of this symbol, i.e. the "symbol itself".
	 */
	
	const string& getName() const;
	
	/**
	 * Provides the arity of this symbol.
	 */
	int getArity() const;
	
	/**
	 * Provides a hash value for this symbol
	 */
	
	size_t hash() const;
	
	// some operator declarations
	
	bool operator==(const Symbol &s) const;
	bool operator!=(const Symbol &s) const;
	bool operator<(const Symbol &s) const;
	bool operator<=(const Symbol &s) const;
	bool operator>(const Symbol &s) const;
	bool operator>=(const Symbol &s) const;
	
private:

	/**
	 * The name of the operation symbol
	 */
	const string name;
	/**
	 * The arity of this symbol. Valid values are integers greater or equal to 0
	 */
	const int arity;
};

// ------------------- inline definitions ----------------------------------

inline size_t Symbol::hash() const {
	return __stl_hash_string(name.c_str());
}

inline bool Symbol::operator==(const Symbol &s) const {
	return name == s.name;
}
inline bool Symbol::operator!=(const Symbol &s) const {
	return name != s.name;
}
inline bool Symbol::operator<(const Symbol &s) const {
	return name < name;
}
inline bool Symbol::operator<=(const Symbol &s) const {
	return name <= name;
}
inline bool Symbol::operator>(const Symbol &s) const {
	return name > name;
}
inline bool Symbol::operator>=(const Symbol &s) const {
	return name >= name;
}
	
inline const string& Symbol::getName() const {
	return name;
}

inline int Symbol::getArity() const {
	return arity;
}

// --------------------- end of inline definitions -------------------------

#endif /*SYMBOL_H_*/
