#ifndef TERM_H_
#define TERM_H_

#include "Symbol.h"

#include <string>
#include <list>

using namespace std;

/**
 * This class represents a first order term without variables.
 * 
 * @author Patrick Bahr
 */
class Term
{	
	
public:
	/**
	 * Constructs a term representation having the given symbol and the given arguments.
	 * 
	 * @param symbol the operation symbol of the term
	 * @param args the arguments of the operation. Of course the number of arguments for the given
	 * operation symbol must match it's arity.
	 */
	Term(Symbol const* symbol, Term** args);
	/**
	 * This constructor is just needed to be compatible with the CCTerm constructor.
	 * This constructor is used by the parsing methods in parsing.h.
	 */

	Term(Symbol const* symbol, Term** args,Term* merge);
	virtual ~Term();
	
	/**
	 * Generates a human readable string representation of this term
	 */ 
	
	string toString() const;
	
	/**
	 * Provides the head symbol of this term.
	 */
	
	Symbol const* getSymbol() const;
	
	/**
	 * Provides an array containing the arguments of this term's head
	 * operation.
	 */
	
	Term**const getArgs() const;
	
	/**
	 * Provides the indexth argument of this term's head operation.
	 * This is equal to getArgs()[index].
	 */
	Term* operator[](int index) const;
	
	/**
	 * Provides the size of this term
	 */
	int size() const;
	
	
protected:

	/**
	 * The head operation symbol of this term
	 */
	
	Symbol const*const symbol;
	
	/**
	 * The arguments of the head operational symbol.
	 */
	Term**const args;
};

// ------------------------ inline definitions -------------------------

inline int Term::size() const{
	int s = 1;
	for(int i=0;i<symbol->getArity();i++) {
		s += args[i]->size();
	}
	return s;
}

inline Term* Term::operator[](int index) const {
	return args[index];
}

inline Symbol const* Term::getSymbol() const {
	return symbol;
}
	
inline Term**const Term::getArgs() const {
	return args;
}


#endif /*TERM_H_*/
