#include "Theory.h"

Theory::Theory()
{
}

Theory::~Theory()
{
	delete [] axioms;
}

string Theory::printSignatureTable() const {
	stringstream ret;
	for(SignatureTable::const_iterator it = signatureTable.begin();it != signatureTable.end();it++) {
		ret << it->first->toString() << it->second->getEqClass() << endl;
	}
	return ret.str();
}

void Theory::buildClosure() {
	
	Combine combine;
	
	signatureTable = SignatureTable();
	
	// traverse axioms
	for(int i=0;i<axiomsCount;i++) {
		traverse(axioms[i]->getLhs(),combine);
		traverse(axioms[i]->getRhs(),combine);
	}
	Pending pending;
	combineAll(combine,pending);
	combine.clear();
	while(!pending.empty()) {
		for(Pending::iterator it = pending.begin();it != pending.end();it++) {
			insertToSignatureTable(*it,combine);
		}
		pending.clear();
		combineAll(combine,pending);
		combine.clear();
		
	}
}

void Theory::traverse(CCTerm* t,Combine& combine) {
	// initialize signatureTable
	insertToSignatureTable(t,combine);
	// initialize predList
	for(int i=0;i<t->getSymbol()->getArity();i++) {
		(*t)[i]->getEqClass()->getPredList()->push_back(t);
		traverse((*t)[i],combine);
		
	}
}


Theory* Theory::parse(istream& input) {
	Theory* thy = new Theory();
	string line;
	
	list<CCEquation*> eqs;
	int lno = 0;
	while(!input.eof()) {
		lno++;
		getline(input,line);
		int offset = 0;
		try {
			CCEquation* eq = parseEq<CCTerm>(thy->arityTable,line,offset);
			eqs.push_back(eq);
		} catch(ParseException pe) {
			delete thy;
			throw TheoryParseException(pe,line,lno);
		}
	}
	thy->axiomsCount = eqs.size();
	thy->axioms = new CCEquation*[thy->axiomsCount];
	list<CCEquation*>::iterator it = eqs.begin();
	for(int i=0;i<thy->axiomsCount;i++,it++) {
		thy->axioms[i] = *it;
	}
	return thy;	
}

SigTab* Theory::copySigTab() const {
	return new SigTab(signatureTable);
}
