#include "Theory.h"
#include <fstream>
#include <time.h>

using namespace std;


int main(int argc, char* argv[]) {
	if(argc < 2) {
		cout << "usage:  " << argv[0] << "[-c] <axioms-file>" << endl;
		cout << endl;
		cout << "options:" << endl;
		cout << "\t -c \t" << "only compute congruence closure" << endl;
		return 1;
	}
	int fileInd = argc-1;
	ifstream file(argv[fileInd]);
	if(!file) {
		cout << "Cannot open file \"" << argv[fileInd] << "\" !" << endl;
		return 1;
	}
	bool onlyCC = false;
	for(int i= 1;i<fileInd;i++) {
		if(strcmp(argv[i],"-c") == 0) {
			onlyCC = true;
		}
	}
		
	time_t start;
	Theory* thy;
	try {
		start = clock();
		thy = Theory::parse(file);
		time_t end = clock();
		cout << thy->getAxiomsCount() << " equations (total size: " << thy->size() << ") parsed in " << (double)(end-start)/CLOCKS_PER_SEC << "s." << endl;
	} catch(TheoryParseException e) {
		cout << "Cannot parse axiom due to syntax errors:" << endl;
		cout << e.getFullMessage() << endl;
		return 0;
	}
	cout << "Computing congruence closure using ";
	#ifdef WEIGHTED_UNION
		cout << "weighted union." << endl;
	#else
		cout << "unweighted union." << endl;
	#endif
	start = clock();
	thy->buildClosure();
	time_t end = clock();
	cout << "Computed congruence closure in " << (double)(end-start)/CLOCKS_PER_SEC << "s." << endl;
	cout.flush();
	if(onlyCC){
		return 0;
	}
	SigTab* sig = thy->copySigTab();
	delete thy;
	string input;
	while(true) {
		cout << "Type the equation you wish to be checked (just hit enter to leave):" << endl;
		cout.flush();
		getline(cin, input);
		if(input.size() == 0) {
			break; // leave
		}
		start = clock();
		try {
			Equation<Term>* eq = sig->parseEquation(input);
			bool b = sig->decide(eq);
			delete eq;
			time_t end = clock();
			if(b) {
				cout << "holds!" << endl;
			} else {
				cout << "does NOT hold!" << endl;
			}
			cout << "Decided in " << (double)(end-start)/CLOCKS_PER_SEC << "s." << endl;
			cout << endl;
		} catch(ParseException e) {
			cout << e.getFullMessage(input) << endl;
		}
	}
	return 0;
}
