#include "parsing.h"

void skipWS(const string& input, int& offset) {
	while(isspace(input[offset])) {
		offset++;
	}
}

template<class T> T* parseTerm(ArityTable& arityTable, const string& input, int& offset, T* merge) {
	T* ret = NULL; // return value
	skipWS(input,offset);
	// read symbol name
	int start = offset;
	while(isalnum(input[offset])) {
		offset++;
	}
	if(offset == start)  {// no symbol read
		throw ParseException("Symbol expected but found '" + input.substr(offset,1) + "'!",offset);
	}
		
	string sym = input.substr(start,offset-start);
	ArityTable::iterator iter = arityTable.find(sym);
	const Symbol* symbol = NULL;
	if(iter != arityTable.end()) {
		symbol = iter->second;
	}
	 
	//read arguments
	skipWS(input,offset);
	if(input[offset] == '(') {
		if(symbol == NULL) {
			list<T*> args;
			int argsCount = 0;
			do {
				offset++;
				skipWS(input,offset);
				args.push_back(parseTerm<T>(arityTable,input,offset,NULL));
				skipWS(input,offset);
				argsCount++;
			} while (input[offset] == ',');
			skipWS(input,offset);
			// insert new symbol in the arity table
			ArityTable::iterator arrIter = arityTable.find(sym);
			if(arrIter == arityTable.end()) {
				symbol=new Symbol(sym,argsCount);
			} else { // symbol might be found in recursive call of parseTerm
				symbol = arrIter->second;
				int exArity = symbol->getArity();
				if(exArity != argsCount) {
					char str[512];
					sprintf(str, "Found %d argument(s) for %d-ary operation \"%s\"!", argsCount,exArity,sym.c_str()); 
					throw ParseException(str,offset);
				}
			}
			arityTable.insert(make_pair(sym,symbol));
			T** argsArr = new T*[argsCount];
			typename list<T*>::iterator jter = args.begin();
			for(int i=0;i<argsCount;i++) {
				argsArr[i] = *(jter++);
			}
			ret = new T(symbol,argsArr,merge);
		} else {
			T** argsArr = new T*[symbol->getArity()];
			offset++;
			for(int i=0;i<symbol->getArity();i++) {
				if(i != 0 ) {
					if(input[offset] != ',') {
						throw ParseException("Too few arguments for operation \"" + sym +"\"!",offset);
					}
					offset++;
				}
				skipWS(input,offset);
				argsArr[i] = parseTerm<T>(arityTable, input,offset,NULL);
				skipWS(input,offset);
			}
			if(input[offset] == ',') {
				throw ParseException("Too much arguments for operation \"" + sym +"\"!",offset);
			}
			ret = new T(symbol,argsArr,merge);
				
		}
		if(input[offset] != ')') {
			throw ParseException("Closing parathesis missing!",offset);
		}
		offset++;
	} else { // read nullary symbol
		if(symbol == NULL) {
			symbol = new Symbol(sym,0);
			arityTable.insert(make_pair(sym,symbol));
		} else {
			if(symbol->getArity() != 0) {
				throw ParseException("Operation \"" + sym + "\" is not nullary",offset);
			}
		}
		ret = new T(symbol,NULL,merge);
	}
	return ret; 
}

template<class T> Equation<T>* parseEq(ArityTable& arityTable, const string& input, int& offset) {
	T* lhs = parseTerm<T>(arityTable,input, offset,NULL);
	skipWS(input,offset);
	if(input[offset] != '=') {
		throw ParseException("\"=\" expected, but found \""+input.substr(offset,1)+"\"!",offset);
	}
	offset++;
	T* rhs = parseTerm(arityTable,input, offset,lhs);
	return new Equation<T>(lhs,rhs);
}

template CCTerm* parseTerm<CCTerm>(ArityTable& arityTable, const string& input, int& offset, CCTerm* merge);
template Equation<CCTerm>* parseEq(ArityTable& arityTable, const string& input, int& offset);
template Term* parseTerm<Term>(ArityTable& arityTable, const string& input, int& offset, Term* merge);
template Equation<Term>* parseEq(ArityTable& arityTable, const string& input, int& offset);
