#ifndef PARSING_H_
#define PARSING_H_

#include <string>
#include <ext/hash_map>

using namespace std;
using namespace __gnu_cxx;

#include "Equation.h"
#include "CCTerm.h"
#include "ParseException.h"
#include "hash_util.h"

typedef hash_map<string,Symbol const*> ArityTable;

/**
 * Skips all consecutive whitespaces (space, tab, newline) in input starting at position offset. The reference parameter
 * is set to the first position in input being not a whitespace 
 */

void skipWS(const string& input, int& offset);

/**
 * Parses a term from the given string input starting at position offset.
 * 
 * @param input the input string to read from
 * @param offset the offset in the input string to start from
 * @param eqClass the int representation of the equivalence class to put the
 * parsed term into.
 */
template<class T> T* parseTerm(ArityTable& arityTable, const string& input, int& offset, T* merge);


/**
 * Parses an equation consisting of two terms separated by a '=' from the given string input starting
 * at position offset. The arityTable argument is used to keep track of the arity of the operation symbols
 */
template<class T> Equation<T>* parseEq(ArityTable& arityTable, const string& input, int& offset);

#endif /*PARSING_H_*/
